/*! dinero.js 2.0.0-alpha.4 | MIT License | © Sarah Dayan and contributors | https://v2.dinerojs.com */
'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var INVALID_AMOUNT_MESSAGE = 'Amount is invalid.';
var INVALID_SCALE_MESSAGE = 'Scale is invalid.';
var INVALID_RATIOS_MESSAGE = 'Ratios are invalid.';
var UNEQUAL_CURRENCIES_MESSAGE = 'Objects must have the same currency.';

/* eslint-disable functional/no-throw-statement, valid-jsdoc */

/**
 * Assert a condition.
 *
 * @param condition - The condition to verify.
 * @param message - The error message to throw.
 *
 * @throws If the condition isn't met.
 */
function assert(condition, message) {
  if (!condition) {
    throw new Error("[Dinero.js] ".concat(message));
  }
}

/* eslint-disable functional/no-mixed-type, functional/no-return-void, functional/no-expression-statement */
function createDinero(_ref) {
  var calculator = _ref.calculator,
      onCreate = _ref.onCreate;
  return function dinero(_ref2) {
    var amount = _ref2.amount,
        currency = _ref2.currency,
        _ref2$scale = _ref2.scale,
        scale = _ref2$scale === void 0 ? currency.exponent : _ref2$scale;
    onCreate === null || onCreate === void 0 ? void 0 : onCreate({
      amount: amount,
      currency: currency,
      scale: scale
    });
    return {
      calculator: calculator,
      create: dinero,
      toJSON: function toJSON() {
        return {
          amount: amount,
          currency: currency,
          scale: scale
        };
      }
    };
  };
}

function _toArray$1(arr) {
  return _arrayWithHoles$9(arr) || _iterableToArray$1(arr) || _unsupportedIterableToArray$9(arr) || _nonIterableRest$9();
}

function _nonIterableRest$9() {
  throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");
}

function _unsupportedIterableToArray$9(o, minLen) {
  if (!o) return;
  if (typeof o === "string") return _arrayLikeToArray$9(o, minLen);
  var n = Object.prototype.toString.call(o).slice(8, -1);
  if (n === "Object" && o.constructor) n = o.constructor.name;
  if (n === "Map" || n === "Set") return Array.from(o);
  if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray$9(o, minLen);
}

function _arrayLikeToArray$9(arr, len) {
  if (len == null || len > arr.length) len = arr.length;

  for (var i = 0, arr2 = new Array(len); i < len; i++) {
    arr2[i] = arr[i];
  }

  return arr2;
}

function _iterableToArray$1(iter) {
  if (typeof Symbol !== "undefined" && iter[Symbol.iterator] != null || iter["@@iterator"] != null) return Array.from(iter);
}

function _arrayWithHoles$9(arr) {
  if (Array.isArray(arr)) return arr;
}

function currencyEqual(subject, comparator) {
  return subject.code === comparator.code && subject.base === comparator.base && subject.exponent === comparator.exponent;
}

function haveSameCurrency$1(dineroObjects) {
  var _dineroObjects = _toArray$1(dineroObjects),
      firstDinero = _dineroObjects[0],
      otherDineros = _dineroObjects.slice(1);

  var _firstDinero$toJSON = firstDinero.toJSON(),
      comparator = _firstDinero$toJSON.currency;

  return otherDineros.every(function (d) {
    var _d$toJSON = d.toJSON(),
        subject = _d$toJSON.currency;

    return currencyEqual(subject, comparator);
  });
}

/* eslint-disable functional/no-mixed-type */
var ComparisonOperator;

(function (ComparisonOperator) {
  ComparisonOperator[ComparisonOperator["LT"] = -1] = "LT";
  ComparisonOperator[ComparisonOperator["EQ"] = 0] = "EQ";
  ComparisonOperator[ComparisonOperator["GT"] = 1] = "GT";
})(ComparisonOperator || (ComparisonOperator = {}));

/**
 * Returns an equal function.
 *
 * @param calculator - The calculator to use.
 *
 * @returns The equal function.
 */

function equal$2(calculator) {
  return function (subject, comparator) {
    return calculator.compare(subject, comparator) === ComparisonOperator.EQ;
  };
}

/* eslint-disable functional/no-let, functional/no-loop-statement, functional/no-expression-statement */
function countTrailingZeros(calculator) {
  var equalFn = equal$2(calculator);
  return function (input, base) {
    var zero = calculator.zero();
    var i = zero;
    var temp = input;

    while (equalFn(calculator.modulo(temp, base), zero)) {
      temp = calculator.integerDivide(temp, base);
      i = calculator.increment(i);
    }

    return i;
  };
}

/**
 * Returns a greaterThan function.
 *
 * @param calculator - The calculator to use.
 *
 * @returns The greaterThan function.
 */

function greaterThan$1(calculator) {
  return function (subject, comparator) {
    return calculator.compare(subject, comparator) === ComparisonOperator.GT;
  };
}

/**
 * Returns a greaterThanOrEqual function.
 *
 * @param calculator - The calculator to use.
 *
 * @returns The greaterThanOrEqual function.
 */

function greaterThanOrEqual$1(calculator) {
  return function (subject, comparator) {
    return greaterThan$1(calculator)(subject, comparator) || equal$2(calculator)(subject, comparator);
  };
}

/**
 * Returns a lessThan function.
 *
 * @param calculator - The calculator to use.
 *
 * @returns The lessThan function.
 */

function lessThan$1(calculator) {
  return function (subject, comparator) {
    return calculator.compare(subject, comparator) === ComparisonOperator.LT;
  };
}

/* eslint-disable functional/no-let, functional/no-loop-statement, functional/immutable-data, functional/no-expression-statement */
/**
 * Returns a distribute function.
 *
 * @param calculator - The calculator to use.
 *
 * @returns The distribute function.
 */

function distribute(calculator) {
  return function (value, ratios) {
    var equalFn = equal$2(calculator);
    var greaterThanFn = greaterThan$1(calculator);
    var lessThanFn = lessThan$1(calculator);
    var greaterThanOrEqualFn = greaterThanOrEqual$1(calculator);
    var zero = calculator.zero();
    var one = calculator.increment(zero);
    var total = ratios.reduce(function (a, b) {
      return calculator.add(a, b);
    }, zero);

    if (equalFn(total, zero)) {
      return ratios;
    }

    var remainder = value;
    var shares = ratios.map(function (ratio) {
      var share = calculator.integerDivide(calculator.multiply(value, ratio), total) || zero;
      remainder = calculator.subtract(remainder, share);
      return share;
    });
    var isPositive = greaterThanOrEqualFn(value, zero);
    var compare = isPositive ? greaterThanFn : lessThanFn;
    var amount = isPositive ? one : calculator.decrement(zero);
    var i = 0;

    while (compare(remainder, zero)) {
      if (!equalFn(ratios[i], zero)) {
        shares[i] = calculator.add(shares[i], amount);
        remainder = calculator.subtract(remainder, amount);
      }

      i++;
    }

    return shares;
  };
}

function isScaledAmount(amount) {
  return amount === null || amount === void 0 ? void 0 : amount.hasOwnProperty('amount');
}

function getAmountAndScale(value, zero) {
  if (isScaledAmount(value)) {
    var _value$scale;

    return {
      amount: value.amount,
      scale: (_value$scale = value === null || value === void 0 ? void 0 : value.scale) !== null && _value$scale !== void 0 ? _value$scale : zero
    };
  }

  return {
    amount: value,
    scale: zero
  };
}

/**
 * Returns a lessThanOrEqual function.
 *
 * @param calculator - The calculator to use.
 *
 * @returns The lessThanOrEqual function.
 */

function lessThanOrEqual$1(calculator) {
  return function (subject, comparator) {
    return lessThan$1(calculator)(subject, comparator) || equal$2(calculator)(subject, comparator);
  };
}

/**
 * Returns a maximum function.
 *
 * @param calculator - The calculator to use.
 *
 * @returns The maximum function.
 */

function maximum$1(calculator) {
  var lessThanFn = lessThan$1(calculator);
  return function (values) {
    return values.reduce(function (acc, curr) {
      return lessThanFn(acc, curr) ? curr : acc;
    });
  };
}

/**
 * Returns a minimum function.
 *
 * @param calculator - The calculator to use.
 *
 * @returns The minimum function.
 */

function minimum$1(calculator) {
  var greaterThanFn = greaterThan$1(calculator);
  return function (values) {
    return values.reduce(function (acc, curr) {
      return greaterThanFn(acc, curr) ? curr : acc;
    });
  };
}

function transformScale$1(_ref) {
  var calculator = _ref.calculator;
  var greaterThanFn = greaterThan$1(calculator);
  return function transformScaleFn() {
    for (var _len = arguments.length, _ref2 = new Array(_len), _key = 0; _key < _len; _key++) {
      _ref2[_key] = arguments[_key];
    }

    var dineroObject = _ref2[0],
        newScale = _ref2[1];

    var _dineroObject$toJSON = dineroObject.toJSON(),
        amount = _dineroObject$toJSON.amount,
        currency = _dineroObject$toJSON.currency,
        scale = _dineroObject$toJSON.scale;

    var isNewScaleLarger = greaterThanFn(newScale, scale);
    var operation = isNewScaleLarger ? calculator.multiply : calculator.integerDivide;
    var terms = isNewScaleLarger ? [newScale, scale] : [scale, newScale];
    var factor = calculator.power(currency.base, calculator.subtract.apply(calculator, terms));
    return dineroObject.create({
      amount: operation(amount, factor),
      currency: currency,
      scale: newScale
    });
  };
}

function normalizeScale$1(_ref) {
  var calculator = _ref.calculator;
  var maximumFn = maximum$1(calculator);
  var convertScaleFn = transformScale$1({
    calculator: calculator
  });
  var equalFn = equal$2(calculator);
  return function _normalizeScale() {
    for (var _len = arguments.length, _ref2 = new Array(_len), _key = 0; _key < _len; _key++) {
      _ref2[_key] = arguments[_key];
    }

    var dineroObjects = _ref2[0];
    var highestScale = dineroObjects.reduce(function (highest, current) {
      var _current$toJSON = current.toJSON(),
          scale = _current$toJSON.scale;

      return maximumFn([highest, scale]);
    }, calculator.zero());
    return dineroObjects.map(function (d) {
      var _d$toJSON = d.toJSON(),
          scale = _d$toJSON.scale;

      return !equalFn(scale, highestScale) ? convertScaleFn(d, highestScale) : d;
    });
  };
}

function _slicedToArray$7(arr, i) {
  return _arrayWithHoles$8(arr) || _iterableToArrayLimit$7(arr, i) || _unsupportedIterableToArray$8(arr, i) || _nonIterableRest$8();
}

function _nonIterableRest$8() {
  throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");
}

function _unsupportedIterableToArray$8(o, minLen) {
  if (!o) return;
  if (typeof o === "string") return _arrayLikeToArray$8(o, minLen);
  var n = Object.prototype.toString.call(o).slice(8, -1);
  if (n === "Object" && o.constructor) n = o.constructor.name;
  if (n === "Map" || n === "Set") return Array.from(o);
  if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray$8(o, minLen);
}

function _arrayLikeToArray$8(arr, len) {
  if (len == null || len > arr.length) len = arr.length;

  for (var i = 0, arr2 = new Array(len); i < len; i++) {
    arr2[i] = arr[i];
  }

  return arr2;
}

function _iterableToArrayLimit$7(arr, i) {
  var _i = arr && (typeof Symbol !== "undefined" && arr[Symbol.iterator] || arr["@@iterator"]);

  if (_i == null) return;
  var _arr = [];
  var _n = true;
  var _d = false;

  var _s, _e;

  try {
    for (_i = _i.call(arr); !(_n = (_s = _i.next()).done); _n = true) {
      _arr.push(_s.value);

      if (i && _arr.length === i) break;
    }
  } catch (err) {
    _d = true;
    _e = err;
  } finally {
    try {
      if (!_n && _i["return"] != null) _i["return"]();
    } finally {
      if (_d) throw _e;
    }
  }

  return _arr;
}

function _arrayWithHoles$8(arr) {
  if (Array.isArray(arr)) return arr;
}
function unsafeAdd(_ref) {
  var calculator = _ref.calculator;
  return function add() {
    for (var _len = arguments.length, _ref2 = new Array(_len), _key = 0; _key < _len; _key++) {
      _ref2[_key] = arguments[_key];
    }

    var augend = _ref2[0],
        addend = _ref2[1];

    var _augend$toJSON = augend.toJSON(),
        augendAmount = _augend$toJSON.amount,
        currency = _augend$toJSON.currency,
        scale = _augend$toJSON.scale;

    var _addend$toJSON = addend.toJSON(),
        addendAmount = _addend$toJSON.amount;

    var amount = calculator.add(augendAmount, addendAmount);
    return augend.create({
      amount: amount,
      currency: currency,
      scale: scale
    });
  };
}
function safeAdd(_ref3) {
  var calculator = _ref3.calculator;
  var normalizeFn = normalizeScale$1({
    calculator: calculator
  });
  var addFn = unsafeAdd({
    calculator: calculator
  });
  return function add() {
    for (var _len2 = arguments.length, _ref4 = new Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
      _ref4[_key2] = arguments[_key2];
    }

    var augend = _ref4[0],
        addend = _ref4[1];
    var condition = haveSameCurrency$1([augend, addend]);
    assert(condition, UNEQUAL_CURRENCIES_MESSAGE);

    var _normalizeFn = normalizeFn([augend, addend]),
        _normalizeFn2 = _slicedToArray$7(_normalizeFn, 2),
        newAugend = _normalizeFn2[0],
        newAddend = _normalizeFn2[1];

    return addFn(newAugend, newAddend);
  };
}

/* eslint-disable functional/no-expression-statement */

function unsafeAllocate(_ref) {
  var calculator = _ref.calculator;
  return function allocate() {
    for (var _len = arguments.length, _ref2 = new Array(_len), _key = 0; _key < _len; _key++) {
      _ref2[_key] = arguments[_key];
    }

    var dineroObject = _ref2[0],
        ratios = _ref2[1];

    var _dineroObject$toJSON = dineroObject.toJSON(),
        amount = _dineroObject$toJSON.amount,
        currency = _dineroObject$toJSON.currency,
        scale = _dineroObject$toJSON.scale;

    var distributeFn = distribute(calculator);
    var shares = distributeFn(amount, ratios.map(function (ratio) {
      return ratio.amount;
    }));
    return shares.map(function (share) {
      return dineroObject.create({
        amount: share,
        currency: currency,
        scale: scale
      });
    });
  };
}

function safeAllocate(_ref3) {
  var calculator = _ref3.calculator;
  var allocateFn = unsafeAllocate({
    calculator: calculator
  });
  var greaterThanOrEqualFn = greaterThanOrEqual$1(calculator);
  var greaterThanFn = greaterThan$1(calculator);
  var convertScaleFn = transformScale$1({
    calculator: calculator
  });
  var maximumFn = maximum$1(calculator);
  var equalFn = equal$2(calculator);
  var zero = calculator.zero();
  var ten = new Array(10).fill(null).reduce(calculator.increment, zero);
  return function allocate() {
    for (var _len2 = arguments.length, _ref4 = new Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
      _ref4[_key2] = arguments[_key2];
    }

    var dineroObject = _ref4[0],
        ratios = _ref4[1];
    var hasRatios = ratios.length > 0;
    var scaledRatios = ratios.map(function (ratio) {
      return getAmountAndScale(ratio, zero);
    });
    var highestRatioScale = hasRatios ? maximumFn(scaledRatios.map(function (_ref5) {
      var scale = _ref5.scale;
      return scale;
    })) : zero;
    var normalizedRatios = scaledRatios.map(function (_ref6) {
      var amount = _ref6.amount,
          scale = _ref6.scale;
      var factor = equalFn(scale, highestRatioScale) ? zero : calculator.subtract(highestRatioScale, scale);
      return {
        amount: calculator.multiply(amount, calculator.power(ten, factor)),
        scale: scale
      };
    });
    var hasOnlyPositiveRatios = normalizedRatios.every(function (_ref7) {
      var amount = _ref7.amount;
      return greaterThanOrEqualFn(amount, zero);
    });
    var hasOneNonZeroRatio = normalizedRatios.some(function (_ref8) {
      var amount = _ref8.amount;
      return greaterThanFn(amount, zero);
    });
    var condition = hasRatios && hasOnlyPositiveRatios && hasOneNonZeroRatio;
    assert(condition, INVALID_RATIOS_MESSAGE);

    var _dineroObject$toJSON2 = dineroObject.toJSON(),
        scale = _dineroObject$toJSON2.scale;

    var newScale = calculator.add(scale, highestRatioScale);
    return allocateFn(convertScaleFn(dineroObject, newScale), normalizedRatios);
  };
}

function convert$1(_ref) {
  var calculator = _ref.calculator;
  var convertScaleFn = transformScale$1({
    calculator: calculator
  });
  var maximumFn = maximum$1(calculator);
  var zero = calculator.zero();
  return function convertFn() {
    for (var _len = arguments.length, _ref2 = new Array(_len), _key = 0; _key < _len; _key++) {
      _ref2[_key] = arguments[_key];
    }

    var dineroObject = _ref2[0],
        newCurrency = _ref2[1],
        rates = _ref2[2];
    var rate = rates[newCurrency.code];

    var _dineroObject$toJSON = dineroObject.toJSON(),
        amount = _dineroObject$toJSON.amount,
        scale = _dineroObject$toJSON.scale;

    var _getAmountAndScale = getAmountAndScale(rate, zero),
        rateAmount = _getAmountAndScale.amount,
        rateScale = _getAmountAndScale.scale;

    var newScale = calculator.add(scale, rateScale);
    return convertScaleFn(dineroObject.create({
      amount: calculator.multiply(amount, rateAmount),
      currency: newCurrency,
      scale: newScale
    }), maximumFn([newScale, newCurrency.exponent]));
  };
}

function _toArray(arr) {
  return _arrayWithHoles$7(arr) || _iterableToArray(arr) || _unsupportedIterableToArray$7(arr) || _nonIterableRest$7();
}

function _nonIterableRest$7() {
  throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");
}

function _unsupportedIterableToArray$7(o, minLen) {
  if (!o) return;
  if (typeof o === "string") return _arrayLikeToArray$7(o, minLen);
  var n = Object.prototype.toString.call(o).slice(8, -1);
  if (n === "Object" && o.constructor) n = o.constructor.name;
  if (n === "Map" || n === "Set") return Array.from(o);
  if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray$7(o, minLen);
}

function _arrayLikeToArray$7(arr, len) {
  if (len == null || len > arr.length) len = arr.length;

  for (var i = 0, arr2 = new Array(len); i < len; i++) {
    arr2[i] = arr[i];
  }

  return arr2;
}

function _iterableToArray(iter) {
  if (typeof Symbol !== "undefined" && iter[Symbol.iterator] != null || iter["@@iterator"] != null) return Array.from(iter);
}

function _arrayWithHoles$7(arr) {
  if (Array.isArray(arr)) return arr;
}
function haveSameAmount$1(_ref) {
  var calculator = _ref.calculator;
  var normalizeFn = normalizeScale$1({
    calculator: calculator
  });
  var equalFn = equal$2(calculator);
  return function _haveSameAmount() {
    for (var _len = arguments.length, _ref2 = new Array(_len), _key = 0; _key < _len; _key++) {
      _ref2[_key] = arguments[_key];
    }

    var dineroObjects = _ref2[0];

    var _normalizeFn = normalizeFn(dineroObjects),
        _normalizeFn2 = _toArray(_normalizeFn),
        firstDinero = _normalizeFn2[0],
        otherDineros = _normalizeFn2.slice(1);

    var _firstDinero$toJSON = firstDinero.toJSON(),
        comparatorAmount = _firstDinero$toJSON.amount;

    return otherDineros.every(function (d) {
      var _d$toJSON = d.toJSON(),
          subjectAmount = _d$toJSON.amount;

      return equalFn(subjectAmount, comparatorAmount);
    });
  };
}

function equal$1(_ref) {
  var calculator = _ref.calculator;
  return function _equal() {
    for (var _len = arguments.length, _ref2 = new Array(_len), _key = 0; _key < _len; _key++) {
      _ref2[_key] = arguments[_key];
    }

    var dineroObject = _ref2[0],
        comparator = _ref2[1];
    return haveSameAmount$1({
      calculator: calculator
    })([dineroObject, comparator]) && haveSameCurrency$1([dineroObject, comparator]);
  };
}

function _slicedToArray$6(arr, i) {
  return _arrayWithHoles$6(arr) || _iterableToArrayLimit$6(arr, i) || _unsupportedIterableToArray$6(arr, i) || _nonIterableRest$6();
}

function _nonIterableRest$6() {
  throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");
}

function _unsupportedIterableToArray$6(o, minLen) {
  if (!o) return;
  if (typeof o === "string") return _arrayLikeToArray$6(o, minLen);
  var n = Object.prototype.toString.call(o).slice(8, -1);
  if (n === "Object" && o.constructor) n = o.constructor.name;
  if (n === "Map" || n === "Set") return Array.from(o);
  if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray$6(o, minLen);
}

function _arrayLikeToArray$6(arr, len) {
  if (len == null || len > arr.length) len = arr.length;

  for (var i = 0, arr2 = new Array(len); i < len; i++) {
    arr2[i] = arr[i];
  }

  return arr2;
}

function _iterableToArrayLimit$6(arr, i) {
  var _i = arr && (typeof Symbol !== "undefined" && arr[Symbol.iterator] || arr["@@iterator"]);

  if (_i == null) return;
  var _arr = [];
  var _n = true;
  var _d = false;

  var _s, _e;

  try {
    for (_i = _i.call(arr); !(_n = (_s = _i.next()).done); _n = true) {
      _arr.push(_s.value);

      if (i && _arr.length === i) break;
    }
  } catch (err) {
    _d = true;
    _e = err;
  } finally {
    try {
      if (!_n && _i["return"] != null) _i["return"]();
    } finally {
      if (_d) throw _e;
    }
  }

  return _arr;
}

function _arrayWithHoles$6(arr) {
  if (Array.isArray(arr)) return arr;
}
function unsafeGreaterThan(_ref) {
  var calculator = _ref.calculator;
  var greaterThanFn = greaterThan$1(calculator);
  return function greaterThan() {
    for (var _len = arguments.length, _ref2 = new Array(_len), _key = 0; _key < _len; _key++) {
      _ref2[_key] = arguments[_key];
    }

    var dineroObject = _ref2[0],
        comparator = _ref2[1];
    var dineroObjects = [dineroObject, comparator];

    var _dineroObjects$map = dineroObjects.map(function (d) {
      var _d$toJSON = d.toJSON(),
          amount = _d$toJSON.amount;

      return amount;
    }),
        _dineroObjects$map2 = _slicedToArray$6(_dineroObjects$map, 2),
        subjectAmount = _dineroObjects$map2[0],
        comparatorAmount = _dineroObjects$map2[1];

    return greaterThanFn(subjectAmount, comparatorAmount);
  };
}
function safeGreaterThan(_ref3) {
  var calculator = _ref3.calculator;
  var normalizeFn = normalizeScale$1({
    calculator: calculator
  });
  var greaterThanFn = unsafeGreaterThan({
    calculator: calculator
  });
  return function greaterThan() {
    for (var _len2 = arguments.length, _ref4 = new Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
      _ref4[_key2] = arguments[_key2];
    }

    var dineroObject = _ref4[0],
        comparator = _ref4[1];
    var condition = haveSameCurrency$1([dineroObject, comparator]);
    assert(condition, UNEQUAL_CURRENCIES_MESSAGE);

    var _normalizeFn = normalizeFn([dineroObject, comparator]),
        _normalizeFn2 = _slicedToArray$6(_normalizeFn, 2),
        subjectAmount = _normalizeFn2[0],
        comparatorAmount = _normalizeFn2[1];

    return greaterThanFn(subjectAmount, comparatorAmount);
  };
}

function _slicedToArray$5(arr, i) {
  return _arrayWithHoles$5(arr) || _iterableToArrayLimit$5(arr, i) || _unsupportedIterableToArray$5(arr, i) || _nonIterableRest$5();
}

function _nonIterableRest$5() {
  throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");
}

function _unsupportedIterableToArray$5(o, minLen) {
  if (!o) return;
  if (typeof o === "string") return _arrayLikeToArray$5(o, minLen);
  var n = Object.prototype.toString.call(o).slice(8, -1);
  if (n === "Object" && o.constructor) n = o.constructor.name;
  if (n === "Map" || n === "Set") return Array.from(o);
  if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray$5(o, minLen);
}

function _arrayLikeToArray$5(arr, len) {
  if (len == null || len > arr.length) len = arr.length;

  for (var i = 0, arr2 = new Array(len); i < len; i++) {
    arr2[i] = arr[i];
  }

  return arr2;
}

function _iterableToArrayLimit$5(arr, i) {
  var _i = arr && (typeof Symbol !== "undefined" && arr[Symbol.iterator] || arr["@@iterator"]);

  if (_i == null) return;
  var _arr = [];
  var _n = true;
  var _d = false;

  var _s, _e;

  try {
    for (_i = _i.call(arr); !(_n = (_s = _i.next()).done); _n = true) {
      _arr.push(_s.value);

      if (i && _arr.length === i) break;
    }
  } catch (err) {
    _d = true;
    _e = err;
  } finally {
    try {
      if (!_n && _i["return"] != null) _i["return"]();
    } finally {
      if (_d) throw _e;
    }
  }

  return _arr;
}

function _arrayWithHoles$5(arr) {
  if (Array.isArray(arr)) return arr;
}
function unsafeGreaterThanOrEqual(_ref) {
  var calculator = _ref.calculator;
  var greaterThanOrEqualFn = greaterThanOrEqual$1(calculator);
  return function greaterThanOrEqual() {
    for (var _len = arguments.length, _ref2 = new Array(_len), _key = 0; _key < _len; _key++) {
      _ref2[_key] = arguments[_key];
    }

    var dineroObject = _ref2[0],
        comparator = _ref2[1];
    var dineroObjects = [dineroObject, comparator];

    var _dineroObjects$map = dineroObjects.map(function (d) {
      var _d$toJSON = d.toJSON(),
          amount = _d$toJSON.amount;

      return amount;
    }),
        _dineroObjects$map2 = _slicedToArray$5(_dineroObjects$map, 2),
        subjectAmount = _dineroObjects$map2[0],
        comparatorAmount = _dineroObjects$map2[1];

    return greaterThanOrEqualFn(subjectAmount, comparatorAmount);
  };
}
function safeGreaterThanOrEqual(_ref3) {
  var calculator = _ref3.calculator;
  var normalizeFn = normalizeScale$1({
    calculator: calculator
  });
  var greaterThanOrEqualFn = unsafeGreaterThanOrEqual({
    calculator: calculator
  });
  return function greaterThanOrEqual() {
    for (var _len2 = arguments.length, _ref4 = new Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
      _ref4[_key2] = arguments[_key2];
    }

    var dineroObject = _ref4[0],
        comparator = _ref4[1];
    var condition = haveSameCurrency$1([dineroObject, comparator]);
    assert(condition, UNEQUAL_CURRENCIES_MESSAGE);

    var _normalizeFn = normalizeFn([dineroObject, comparator]),
        _normalizeFn2 = _slicedToArray$5(_normalizeFn, 2),
        subjectAmount = _normalizeFn2[0],
        comparatorAmount = _normalizeFn2[1];

    return greaterThanOrEqualFn(subjectAmount, comparatorAmount);
  };
}

function hasSubUnits$1(_ref) {
  var calculator = _ref.calculator;
  var equalFn = equal$2(calculator);
  return function _hasSubUnits() {
    for (var _len = arguments.length, _ref2 = new Array(_len), _key = 0; _key < _len; _key++) {
      _ref2[_key] = arguments[_key];
    }

    var dineroObject = _ref2[0];

    var _dineroObject$toJSON = dineroObject.toJSON(),
        amount = _dineroObject$toJSON.amount,
        currency = _dineroObject$toJSON.currency,
        scale = _dineroObject$toJSON.scale;

    return !equalFn(calculator.modulo(amount, calculator.power(currency.base, scale)), calculator.zero());
  };
}

function isNegative$1(_ref) {
  var calculator = _ref.calculator;
  var lessThanFn = lessThan$1(calculator);
  return function _isNegative() {
    for (var _len = arguments.length, _ref2 = new Array(_len), _key = 0; _key < _len; _key++) {
      _ref2[_key] = arguments[_key];
    }

    var dineroObject = _ref2[0];

    var _dineroObject$toJSON = dineroObject.toJSON(),
        amount = _dineroObject$toJSON.amount;

    return lessThanFn(amount, calculator.zero());
  };
}

function isPositive$1(_ref) {
  var calculator = _ref.calculator;
  var greaterThanOrEqualFn = greaterThanOrEqual$1(calculator);
  return function _isPositive() {
    for (var _len = arguments.length, _ref2 = new Array(_len), _key = 0; _key < _len; _key++) {
      _ref2[_key] = arguments[_key];
    }

    var dineroObject = _ref2[0];

    var _dineroObject$toJSON = dineroObject.toJSON(),
        amount = _dineroObject$toJSON.amount;

    return greaterThanOrEqualFn(amount, calculator.zero());
  };
}

function isZero$1(_ref) {
  var calculator = _ref.calculator;
  var equalFn = equal$2(calculator);
  return function _isZero() {
    for (var _len = arguments.length, _ref2 = new Array(_len), _key = 0; _key < _len; _key++) {
      _ref2[_key] = arguments[_key];
    }

    var dineroObject = _ref2[0];

    var _dineroObject$toJSON = dineroObject.toJSON(),
        amount = _dineroObject$toJSON.amount;

    return equalFn(amount, calculator.zero());
  };
}

function _slicedToArray$4(arr, i) {
  return _arrayWithHoles$4(arr) || _iterableToArrayLimit$4(arr, i) || _unsupportedIterableToArray$4(arr, i) || _nonIterableRest$4();
}

function _nonIterableRest$4() {
  throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");
}

function _unsupportedIterableToArray$4(o, minLen) {
  if (!o) return;
  if (typeof o === "string") return _arrayLikeToArray$4(o, minLen);
  var n = Object.prototype.toString.call(o).slice(8, -1);
  if (n === "Object" && o.constructor) n = o.constructor.name;
  if (n === "Map" || n === "Set") return Array.from(o);
  if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray$4(o, minLen);
}

function _arrayLikeToArray$4(arr, len) {
  if (len == null || len > arr.length) len = arr.length;

  for (var i = 0, arr2 = new Array(len); i < len; i++) {
    arr2[i] = arr[i];
  }

  return arr2;
}

function _iterableToArrayLimit$4(arr, i) {
  var _i = arr && (typeof Symbol !== "undefined" && arr[Symbol.iterator] || arr["@@iterator"]);

  if (_i == null) return;
  var _arr = [];
  var _n = true;
  var _d = false;

  var _s, _e;

  try {
    for (_i = _i.call(arr); !(_n = (_s = _i.next()).done); _n = true) {
      _arr.push(_s.value);

      if (i && _arr.length === i) break;
    }
  } catch (err) {
    _d = true;
    _e = err;
  } finally {
    try {
      if (!_n && _i["return"] != null) _i["return"]();
    } finally {
      if (_d) throw _e;
    }
  }

  return _arr;
}

function _arrayWithHoles$4(arr) {
  if (Array.isArray(arr)) return arr;
}
function unsafeLessThan(_ref) {
  var calculator = _ref.calculator;
  var lessThanFn = lessThan$1(calculator);
  return function lessThan() {
    for (var _len = arguments.length, _ref2 = new Array(_len), _key = 0; _key < _len; _key++) {
      _ref2[_key] = arguments[_key];
    }

    var dineroObject = _ref2[0],
        comparator = _ref2[1];
    var dineroObjects = [dineroObject, comparator];

    var _dineroObjects$map = dineroObjects.map(function (d) {
      var _d$toJSON = d.toJSON(),
          amount = _d$toJSON.amount;

      return amount;
    }),
        _dineroObjects$map2 = _slicedToArray$4(_dineroObjects$map, 2),
        subjectAmount = _dineroObjects$map2[0],
        comparatorAmount = _dineroObjects$map2[1];

    return lessThanFn(subjectAmount, comparatorAmount);
  };
}
function safeLessThan(_ref3) {
  var calculator = _ref3.calculator;
  var normalizeFn = normalizeScale$1({
    calculator: calculator
  });
  var lessThanFn = unsafeLessThan({
    calculator: calculator
  });
  return function lessThan() {
    for (var _len2 = arguments.length, _ref4 = new Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
      _ref4[_key2] = arguments[_key2];
    }

    var dineroObject = _ref4[0],
        comparator = _ref4[1];
    var condition = haveSameCurrency$1([dineroObject, comparator]);
    assert(condition, UNEQUAL_CURRENCIES_MESSAGE);

    var _normalizeFn = normalizeFn([dineroObject, comparator]),
        _normalizeFn2 = _slicedToArray$4(_normalizeFn, 2),
        subjectAmount = _normalizeFn2[0],
        comparatorAmount = _normalizeFn2[1];

    return lessThanFn(subjectAmount, comparatorAmount);
  };
}

function _slicedToArray$3(arr, i) {
  return _arrayWithHoles$3(arr) || _iterableToArrayLimit$3(arr, i) || _unsupportedIterableToArray$3(arr, i) || _nonIterableRest$3();
}

function _nonIterableRest$3() {
  throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");
}

function _unsupportedIterableToArray$3(o, minLen) {
  if (!o) return;
  if (typeof o === "string") return _arrayLikeToArray$3(o, minLen);
  var n = Object.prototype.toString.call(o).slice(8, -1);
  if (n === "Object" && o.constructor) n = o.constructor.name;
  if (n === "Map" || n === "Set") return Array.from(o);
  if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray$3(o, minLen);
}

function _arrayLikeToArray$3(arr, len) {
  if (len == null || len > arr.length) len = arr.length;

  for (var i = 0, arr2 = new Array(len); i < len; i++) {
    arr2[i] = arr[i];
  }

  return arr2;
}

function _iterableToArrayLimit$3(arr, i) {
  var _i = arr && (typeof Symbol !== "undefined" && arr[Symbol.iterator] || arr["@@iterator"]);

  if (_i == null) return;
  var _arr = [];
  var _n = true;
  var _d = false;

  var _s, _e;

  try {
    for (_i = _i.call(arr); !(_n = (_s = _i.next()).done); _n = true) {
      _arr.push(_s.value);

      if (i && _arr.length === i) break;
    }
  } catch (err) {
    _d = true;
    _e = err;
  } finally {
    try {
      if (!_n && _i["return"] != null) _i["return"]();
    } finally {
      if (_d) throw _e;
    }
  }

  return _arr;
}

function _arrayWithHoles$3(arr) {
  if (Array.isArray(arr)) return arr;
}
function unsafeLessThanOrEqual(_ref) {
  var calculator = _ref.calculator;
  var lessThanOrEqualFn = lessThanOrEqual$1(calculator);
  return function lessThanOrEqual() {
    for (var _len = arguments.length, _ref2 = new Array(_len), _key = 0; _key < _len; _key++) {
      _ref2[_key] = arguments[_key];
    }

    var dineroObject = _ref2[0],
        comparator = _ref2[1];
    var dineroObjects = [dineroObject, comparator];

    var _dineroObjects$map = dineroObjects.map(function (d) {
      var _d$toJSON = d.toJSON(),
          amount = _d$toJSON.amount;

      return amount;
    }),
        _dineroObjects$map2 = _slicedToArray$3(_dineroObjects$map, 2),
        subjectAmount = _dineroObjects$map2[0],
        comparatorAmount = _dineroObjects$map2[1];

    return lessThanOrEqualFn(subjectAmount, comparatorAmount);
  };
}
function safeLessThanOrEqual(_ref3) {
  var calculator = _ref3.calculator;
  var normalizeFn = normalizeScale$1({
    calculator: calculator
  });
  var lessThanOrEqualFn = unsafeLessThanOrEqual({
    calculator: calculator
  });
  return function lessThanOrEqual() {
    for (var _len2 = arguments.length, _ref4 = new Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
      _ref4[_key2] = arguments[_key2];
    }

    var dineroObject = _ref4[0],
        comparator = _ref4[1];
    var condition = haveSameCurrency$1([dineroObject, comparator]);
    assert(condition, UNEQUAL_CURRENCIES_MESSAGE);

    var _normalizeFn = normalizeFn([dineroObject, comparator]),
        _normalizeFn2 = _slicedToArray$3(_normalizeFn, 2),
        subjectAmount = _normalizeFn2[0],
        comparatorAmount = _normalizeFn2[1];

    return lessThanOrEqualFn(subjectAmount, comparatorAmount);
  };
}

function _slicedToArray$2(arr, i) {
  return _arrayWithHoles$2(arr) || _iterableToArrayLimit$2(arr, i) || _unsupportedIterableToArray$2(arr, i) || _nonIterableRest$2();
}

function _nonIterableRest$2() {
  throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");
}

function _unsupportedIterableToArray$2(o, minLen) {
  if (!o) return;
  if (typeof o === "string") return _arrayLikeToArray$2(o, minLen);
  var n = Object.prototype.toString.call(o).slice(8, -1);
  if (n === "Object" && o.constructor) n = o.constructor.name;
  if (n === "Map" || n === "Set") return Array.from(o);
  if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray$2(o, minLen);
}

function _arrayLikeToArray$2(arr, len) {
  if (len == null || len > arr.length) len = arr.length;

  for (var i = 0, arr2 = new Array(len); i < len; i++) {
    arr2[i] = arr[i];
  }

  return arr2;
}

function _iterableToArrayLimit$2(arr, i) {
  var _i = arr && (typeof Symbol !== "undefined" && arr[Symbol.iterator] || arr["@@iterator"]);

  if (_i == null) return;
  var _arr = [];
  var _n = true;
  var _d = false;

  var _s, _e;

  try {
    for (_i = _i.call(arr); !(_n = (_s = _i.next()).done); _n = true) {
      _arr.push(_s.value);

      if (i && _arr.length === i) break;
    }
  } catch (err) {
    _d = true;
    _e = err;
  } finally {
    try {
      if (!_n && _i["return"] != null) _i["return"]();
    } finally {
      if (_d) throw _e;
    }
  }

  return _arr;
}

function _arrayWithHoles$2(arr) {
  if (Array.isArray(arr)) return arr;
}
function unsafeMaximum(_ref) {
  var calculator = _ref.calculator;
  var maxFn = maximum$1(calculator);
  return function maximum() {
    for (var _len = arguments.length, _ref2 = new Array(_len), _key = 0; _key < _len; _key++) {
      _ref2[_key] = arguments[_key];
    }

    var dineroObjects = _ref2[0];

    var _dineroObjects = _slicedToArray$2(dineroObjects, 1),
        firstDinero = _dineroObjects[0];

    var _firstDinero$toJSON = firstDinero.toJSON(),
        currency = _firstDinero$toJSON.currency,
        scale = _firstDinero$toJSON.scale;

    var amount = maxFn(dineroObjects.map(function (subject) {
      var _subject$toJSON = subject.toJSON(),
          subjectAmount = _subject$toJSON.amount;

      return subjectAmount;
    }));
    return firstDinero.create({
      amount: amount,
      currency: currency,
      scale: scale
    });
  };
}
function safeMaximum(_ref3) {
  var calculator = _ref3.calculator;
  var normalizeFn = normalizeScale$1({
    calculator: calculator
  });
  var maxFn = unsafeMaximum({
    calculator: calculator
  });
  return function maximum() {
    for (var _len2 = arguments.length, _ref4 = new Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
      _ref4[_key2] = arguments[_key2];
    }

    var dineroObjects = _ref4[0];
    var condition = haveSameCurrency$1(dineroObjects);
    assert(condition, UNEQUAL_CURRENCIES_MESSAGE);
    var normalizedDineroObjects = normalizeFn(dineroObjects);
    return maxFn(normalizedDineroObjects);
  };
}

function _slicedToArray$1(arr, i) {
  return _arrayWithHoles$1(arr) || _iterableToArrayLimit$1(arr, i) || _unsupportedIterableToArray$1(arr, i) || _nonIterableRest$1();
}

function _nonIterableRest$1() {
  throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");
}

function _unsupportedIterableToArray$1(o, minLen) {
  if (!o) return;
  if (typeof o === "string") return _arrayLikeToArray$1(o, minLen);
  var n = Object.prototype.toString.call(o).slice(8, -1);
  if (n === "Object" && o.constructor) n = o.constructor.name;
  if (n === "Map" || n === "Set") return Array.from(o);
  if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray$1(o, minLen);
}

function _arrayLikeToArray$1(arr, len) {
  if (len == null || len > arr.length) len = arr.length;

  for (var i = 0, arr2 = new Array(len); i < len; i++) {
    arr2[i] = arr[i];
  }

  return arr2;
}

function _iterableToArrayLimit$1(arr, i) {
  var _i = arr && (typeof Symbol !== "undefined" && arr[Symbol.iterator] || arr["@@iterator"]);

  if (_i == null) return;
  var _arr = [];
  var _n = true;
  var _d = false;

  var _s, _e;

  try {
    for (_i = _i.call(arr); !(_n = (_s = _i.next()).done); _n = true) {
      _arr.push(_s.value);

      if (i && _arr.length === i) break;
    }
  } catch (err) {
    _d = true;
    _e = err;
  } finally {
    try {
      if (!_n && _i["return"] != null) _i["return"]();
    } finally {
      if (_d) throw _e;
    }
  }

  return _arr;
}

function _arrayWithHoles$1(arr) {
  if (Array.isArray(arr)) return arr;
}
function unsafeMinimum(_ref) {
  var calculator = _ref.calculator;
  var minFn = minimum$1(calculator);
  return function minimum() {
    for (var _len = arguments.length, _ref2 = new Array(_len), _key = 0; _key < _len; _key++) {
      _ref2[_key] = arguments[_key];
    }

    var dineroObjects = _ref2[0];

    var _dineroObjects = _slicedToArray$1(dineroObjects, 1),
        firstDinero = _dineroObjects[0];

    var _firstDinero$toJSON = firstDinero.toJSON(),
        currency = _firstDinero$toJSON.currency,
        scale = _firstDinero$toJSON.scale;

    var amount = minFn(dineroObjects.map(function (subject) {
      var _subject$toJSON = subject.toJSON(),
          subjectAmount = _subject$toJSON.amount;

      return subjectAmount;
    }));
    return firstDinero.create({
      amount: amount,
      currency: currency,
      scale: scale
    });
  };
}
function safeMinimum(_ref3) {
  var calculator = _ref3.calculator;
  var normalizeFn = normalizeScale$1({
    calculator: calculator
  });
  var minFn = unsafeMinimum({
    calculator: calculator
  });
  return function maximum() {
    for (var _len2 = arguments.length, _ref4 = new Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
      _ref4[_key2] = arguments[_key2];
    }

    var dineroObjects = _ref4[0];
    var condition = haveSameCurrency$1(dineroObjects);
    assert(condition, UNEQUAL_CURRENCIES_MESSAGE);
    var normalizedDineroObjects = normalizeFn(dineroObjects);
    return minFn(normalizedDineroObjects);
  };
}

function multiply$2(_ref) {
  var calculator = _ref.calculator;
  var convertScaleFn = transformScale$1({
    calculator: calculator
  });
  var zero = calculator.zero();
  return function multiplyFn() {
    for (var _len = arguments.length, _ref2 = new Array(_len), _key = 0; _key < _len; _key++) {
      _ref2[_key] = arguments[_key];
    }

    var multiplicand = _ref2[0],
        multiplier = _ref2[1];

    var _multiplicand$toJSON = multiplicand.toJSON(),
        amount = _multiplicand$toJSON.amount,
        currency = _multiplicand$toJSON.currency,
        scale = _multiplicand$toJSON.scale;

    var _getAmountAndScale = getAmountAndScale(multiplier, zero),
        multiplierAmount = _getAmountAndScale.amount,
        multiplierScale = _getAmountAndScale.scale;

    var newScale = calculator.add(scale, multiplierScale);
    return convertScaleFn(multiplicand.create({
      amount: calculator.multiply(amount, multiplierAmount),
      currency: currency,
      scale: newScale
    }), newScale);
  };
}

function _slicedToArray(arr, i) {
  return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _unsupportedIterableToArray(arr, i) || _nonIterableRest();
}

function _nonIterableRest() {
  throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");
}

function _unsupportedIterableToArray(o, minLen) {
  if (!o) return;
  if (typeof o === "string") return _arrayLikeToArray(o, minLen);
  var n = Object.prototype.toString.call(o).slice(8, -1);
  if (n === "Object" && o.constructor) n = o.constructor.name;
  if (n === "Map" || n === "Set") return Array.from(o);
  if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen);
}

function _arrayLikeToArray(arr, len) {
  if (len == null || len > arr.length) len = arr.length;

  for (var i = 0, arr2 = new Array(len); i < len; i++) {
    arr2[i] = arr[i];
  }

  return arr2;
}

function _iterableToArrayLimit(arr, i) {
  var _i = arr && (typeof Symbol !== "undefined" && arr[Symbol.iterator] || arr["@@iterator"]);

  if (_i == null) return;
  var _arr = [];
  var _n = true;
  var _d = false;

  var _s, _e;

  try {
    for (_i = _i.call(arr); !(_n = (_s = _i.next()).done); _n = true) {
      _arr.push(_s.value);

      if (i && _arr.length === i) break;
    }
  } catch (err) {
    _d = true;
    _e = err;
  } finally {
    try {
      if (!_n && _i["return"] != null) _i["return"]();
    } finally {
      if (_d) throw _e;
    }
  }

  return _arr;
}

function _arrayWithHoles(arr) {
  if (Array.isArray(arr)) return arr;
}
function unsafeSubtract(_ref) {
  var calculator = _ref.calculator;
  return function subtract() {
    for (var _len = arguments.length, _ref2 = new Array(_len), _key = 0; _key < _len; _key++) {
      _ref2[_key] = arguments[_key];
    }

    var minuend = _ref2[0],
        subtrahend = _ref2[1];

    var _minuend$toJSON = minuend.toJSON(),
        minuendAmount = _minuend$toJSON.amount,
        currency = _minuend$toJSON.currency,
        scale = _minuend$toJSON.scale;

    var _subtrahend$toJSON = subtrahend.toJSON(),
        subtrahendAmount = _subtrahend$toJSON.amount;

    var amount = calculator.subtract(minuendAmount, subtrahendAmount);
    return minuend.create({
      amount: amount,
      currency: currency,
      scale: scale
    });
  };
}
function safeSubtract(_ref3) {
  var calculator = _ref3.calculator;
  var normalizeFn = normalizeScale$1({
    calculator: calculator
  });
  var subtractFn = unsafeSubtract({
    calculator: calculator
  });
  return function subtract() {
    for (var _len2 = arguments.length, _ref4 = new Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
      _ref4[_key2] = arguments[_key2];
    }

    var minuend = _ref4[0],
        subtrahend = _ref4[1];
    var condition = haveSameCurrency$1([minuend, subtrahend]);
    assert(condition, UNEQUAL_CURRENCIES_MESSAGE);

    var _normalizeFn = normalizeFn([minuend, subtrahend]),
        _normalizeFn2 = _slicedToArray(_normalizeFn, 2),
        newMinuend = _normalizeFn2[0],
        newSubtrahend = _normalizeFn2[1];

    return subtractFn(newMinuend, newSubtrahend);
  };
}

function toUnit$1(_ref) {
  var calculator = _ref.calculator;
  return function toUnitFn() {
    var _options$digits;

    for (var _len = arguments.length, _ref2 = new Array(_len), _key = 0; _key < _len; _key++) {
      _ref2[_key] = arguments[_key];
    }

    var dineroObject = _ref2[0],
        options = _ref2[1];
    var round = (options === null || options === void 0 ? void 0 : options.round) || identity;

    var _dineroObject$toJSON = dineroObject.toJSON(),
        amount = _dineroObject$toJSON.amount,
        currency = _dineroObject$toJSON.currency,
        scale = _dineroObject$toJSON.scale;

    var power = calculator.power,
        toNumber = calculator.toNumber;
    var toUnitFactor = toNumber(power(currency.base, scale));
    var factor = toNumber(power(currency.base, (_options$digits = options === null || options === void 0 ? void 0 : options.digits) !== null && _options$digits !== void 0 ? _options$digits : scale));
    return round(toNumber(amount) / toUnitFactor * factor) / factor;
  };
}

function identity(value) {
  return value;
}

function toFormat$1(_ref) {
  var calculator = _ref.calculator;
  var toUnitFn = toUnit$1({
    calculator: calculator
  });
  return function toFormatFn() {
    for (var _len = arguments.length, _ref2 = new Array(_len), _key = 0; _key < _len; _key++) {
      _ref2[_key] = arguments[_key];
    }

    var dineroObject = _ref2[0],
        transformer = _ref2[1];

    var _dineroObject$toJSON = dineroObject.toJSON(),
        currency = _dineroObject$toJSON.currency,
        scale = _dineroObject$toJSON.scale;

    var amount = toUnitFn(dineroObject, {
      digits: scale
    });
    return transformer({
      amount: amount,
      currency: currency,
      dineroObject: dineroObject
    });
  };
}

function toSnapshot$1(dineroObject) {
  return dineroObject.toJSON();
}

function trimScale$1(_ref) {
  var calculator = _ref.calculator;
  var countTrailingZerosFn = countTrailingZeros(calculator);
  var equalFn = equal$2(calculator);
  var maximumFn = maximum$1(calculator);
  var transformScaleFn = transformScale$1({
    calculator: calculator
  });
  return function trimScaleFn() {
    for (var _len = arguments.length, _ref2 = new Array(_len), _key = 0; _key < _len; _key++) {
      _ref2[_key] = arguments[_key];
    }

    var dineroObject = _ref2[0];

    var _dineroObject$toJSON = dineroObject.toJSON(),
        amount = _dineroObject$toJSON.amount,
        currency = _dineroObject$toJSON.currency,
        scale = _dineroObject$toJSON.scale;

    var base = currency.base,
        exponent = currency.exponent;
    var trailingZerosLength = countTrailingZerosFn(amount, base);
    var difference = calculator.subtract(scale, trailingZerosLength);
    var newScale = maximumFn([difference, exponent]);

    if (equalFn(newScale, scale)) {
      return dineroObject;
    }

    return transformScaleFn(dineroObject, newScale);
  };
}

/**
 * Add up the passed Dinero objects.
 *
 * @param augend - The Dinero object to add to.
 * @param addend - The Dinero object to add.
 *
 * @returns A new Dinero object.
 */
function add$1() {
  for (var _len = arguments.length, _ref = new Array(_len), _key = 0; _key < _len; _key++) {
    _ref[_key] = arguments[_key];
  }

  var augend = _ref[0],
      addend = _ref[1];
  var calculator = augend.calculator;
  var addFn = safeAdd({
    calculator: calculator
  });
  return addFn(augend, addend);
}

/**
 * Distribute the amount of a Dinero object across a list of ratios.
 *
 * @param dineroObject - The Dinero object to allocate from.
 * @param ratios - The ratios to allocate the amount to.
 *
 * @returns A new Dinero object.
 */
function allocate() {
  for (var _len = arguments.length, _ref = new Array(_len), _key = 0; _key < _len; _key++) {
    _ref[_key] = arguments[_key];
  }

  var dineroObject = _ref[0],
      ratios = _ref[1];
  var calculator = dineroObject.calculator;
  var allocateFn = safeAllocate({
    calculator: calculator
  });
  return allocateFn(dineroObject, ratios);
}

/**
 * Convert a Dinero object to another currency.
 *
 * @param dineroObject - The Dinero object to format.
 * @param newCurrency - The currency to convert to.
 * @param rates - The rates to convert with.
 *
 * @returns A converted Dinero object.
 */
function convert() {
  for (var _len = arguments.length, _ref = new Array(_len), _key = 0; _key < _len; _key++) {
    _ref[_key] = arguments[_key];
  }

  var dineroObject = _ref[0],
      newCurrency = _ref[1],
      rates = _ref[2];
  var calculator = dineroObject.calculator;
  var converter = convert$1({
    calculator: calculator
  });
  return converter(dineroObject, newCurrency, rates);
}

/**
 * Check whether the value of a Dinero object is equal to another.
 *
 * @param dineroObject - The first Dinero object to compare.
 * @param comparator - The second Dinero object to compare.
 *
 * @returns Whether the Dinero objects are equal.
 */
function equal() {
  for (var _len = arguments.length, _ref = new Array(_len), _key = 0; _key < _len; _key++) {
    _ref[_key] = arguments[_key];
  }

  var dineroObject = _ref[0],
      comparator = _ref[1];
  var calculator = dineroObject.calculator;
  var equalFn = equal$1({
    calculator: calculator
  });
  return equalFn(dineroObject, comparator);
}

/**
 * Check whether the value of a Dinero object is greater than another.
 *
 * @param dineroObject - The Dinero object to compare.
 * @param comparator - The Dinero object to compare to.
 *
 * @returns Whether the Dinero to compare is greater than the other.
 */
function greaterThan() {
  for (var _len = arguments.length, _ref = new Array(_len), _key = 0; _key < _len; _key++) {
    _ref[_key] = arguments[_key];
  }

  var dineroObject = _ref[0],
      comparator = _ref[1];
  var calculator = dineroObject.calculator;
  var greaterThanFn = safeGreaterThan({
    calculator: calculator
  });
  return greaterThanFn(dineroObject, comparator);
}

/**
 * Check whether the value of a Dinero object is greater than or equal another.
 *
 * @param dineroObject - The Dinero object to compare.
 * @param comparator - The Dinero object to compare to.
 *
 * @returns Whether the Dinero to compare is greater than or equal the other.
 */
function greaterThanOrEqual() {
  for (var _len = arguments.length, _ref = new Array(_len), _key = 0; _key < _len; _key++) {
    _ref[_key] = arguments[_key];
  }

  var dineroObject = _ref[0],
      comparator = _ref[1];
  var calculator = dineroObject.calculator;
  var greaterThanOrEqualFn = safeGreaterThanOrEqual({
    calculator: calculator
  });
  return greaterThanOrEqualFn(dineroObject, comparator);
}

/**
 * Check whether a Dinero object has minor currency units.
 *
 * @param dineroObject - The Dinero object to check.
 *
 * @returns Whether the Dinero object has minor currency units.
 */
function hasSubUnits() {
  for (var _len = arguments.length, _ref = new Array(_len), _key = 0; _key < _len; _key++) {
    _ref[_key] = arguments[_key];
  }

  var dineroObject = _ref[0];
  var calculator = dineroObject.calculator;
  var hasSubUnitsFn = hasSubUnits$1({
    calculator: calculator
  });
  return hasSubUnitsFn(dineroObject);
}

/**
 * Check whether a set of Dinero objects have the same amount.
 *
 * @param dineroObjects - The Dinero objects to compare.
 *
 * @returns Whether the Dinero objects have the same amount.
 */
function haveSameAmount() {
  for (var _len = arguments.length, _ref = new Array(_len), _key = 0; _key < _len; _key++) {
    _ref[_key] = arguments[_key];
  }

  var dineroObjects = _ref[0];
  var calculator = dineroObjects[0].calculator;
  var haveSameAmountFn = haveSameAmount$1({
    calculator: calculator
  });
  return haveSameAmountFn(dineroObjects);
}

/**
 * Check whether a set of Dinero objects have the same currency.
 *
 * @param dineroObjects - The Dinero objects to compare.
 *
 * @returns Whether the Dinero objects have the same currency.
 */

var haveSameCurrency = haveSameCurrency$1;

/**
 * Check whether a Dinero object is negative.
 *
 * @param dineroObject - The Dinero object to check.
 *
 * @returns Whether the Dinero object is negative.
 */
function isNegative() {
  for (var _len = arguments.length, _ref = new Array(_len), _key = 0; _key < _len; _key++) {
    _ref[_key] = arguments[_key];
  }

  var dineroObject = _ref[0];
  var calculator = dineroObject.calculator;
  var isNegativeFn = isNegative$1({
    calculator: calculator
  });
  return isNegativeFn(dineroObject);
}

/**
 * Check whether a Dinero object is positive.
 *
 * @param dineroObject - The Dinero object to check.
 *
 * @returns Whether the Dinero object is positive.
 */
function isPositive() {
  for (var _len = arguments.length, _ref = new Array(_len), _key = 0; _key < _len; _key++) {
    _ref[_key] = arguments[_key];
  }

  var dineroObject = _ref[0];
  var calculator = dineroObject.calculator;
  var isPositiveFn = isPositive$1({
    calculator: calculator
  });
  return isPositiveFn(dineroObject);
}

/**
 * Check whether the value of a Dinero object is zero.
 *
 * @param dineroObject - The Dinero object to check.
 *
 * @returns Whether the value of a Dinero object is zero.
 */
function isZero() {
  for (var _len = arguments.length, _ref = new Array(_len), _key = 0; _key < _len; _key++) {
    _ref[_key] = arguments[_key];
  }

  var dineroObject = _ref[0];
  var calculator = dineroObject.calculator;
  var isZeroFn = isZero$1({
    calculator: calculator
  });
  return isZeroFn(dineroObject);
}

/**
 * Check whether the value of a Dinero object is lesser than another.
 *
 * @param dineroObject - The Dinero object to compare.
 * @param comparator - The Dinero object to compare to.
 *
 * @returns Whether the Dinero to compare is lesser than the other.
 */
function lessThan() {
  for (var _len = arguments.length, _ref = new Array(_len), _key = 0; _key < _len; _key++) {
    _ref[_key] = arguments[_key];
  }

  var dineroObject = _ref[0],
      comparator = _ref[1];
  var calculator = dineroObject.calculator;
  var lessThanFn = safeLessThan({
    calculator: calculator
  });
  return lessThanFn(dineroObject, comparator);
}

/**
 * Check whether the value of a Dinero object is lesser than or equal to another.
 *
 * @param dineroObject - The Dinero object to compare.
 * @param comparator - The Dinero object to compare to.
 *
 * @returns Whether the Dinero to compare is lesser than or equal to the other.
 */
function lessThanOrEqual() {
  for (var _len = arguments.length, _ref = new Array(_len), _key = 0; _key < _len; _key++) {
    _ref[_key] = arguments[_key];
  }

  var dineroObject = _ref[0],
      comparator = _ref[1];
  var calculator = dineroObject.calculator;
  var lessThanOrEqualFn = safeLessThanOrEqual({
    calculator: calculator
  });
  return lessThanOrEqualFn(dineroObject, comparator);
}

/**
 * Get the greatest of the passed Dinero objects.
 *
 * @param dineroObjects - The Dinero objects to maximum.
 *
 * @returns A new Dinero object.
 */
function maximum() {
  for (var _len = arguments.length, _ref = new Array(_len), _key = 0; _key < _len; _key++) {
    _ref[_key] = arguments[_key];
  }

  var dineroObjects = _ref[0];
  var maximumFn = safeMaximum({
    calculator: dineroObjects[0].calculator
  });
  return maximumFn(dineroObjects);
}

/**
 * Get the lowest of the passed Dinero objects.
 *
 * @param dineroObjects - The Dinero objects to minimum.
 *
 * @returns A new Dinero object.
 */
function minimum() {
  for (var _len = arguments.length, _ref = new Array(_len), _key = 0; _key < _len; _key++) {
    _ref[_key] = arguments[_key];
  }

  var dineroObjects = _ref[0];
  var minimumFn = safeMinimum({
    calculator: dineroObjects[0].calculator
  });
  return minimumFn(dineroObjects);
}

/**
 * Multiply the passed Dinero object.
 *
 * @param multiplicand - The Dinero object to multiply.
 * @param multiplier - The number to multiply with.
 *
 * @returns A new Dinero object.
 */
function multiply$1() {
  for (var _len = arguments.length, _ref = new Array(_len), _key = 0; _key < _len; _key++) {
    _ref[_key] = arguments[_key];
  }

  var multiplicand = _ref[0],
      multiplier = _ref[1];
  var calculator = multiplicand.calculator;
  var multiplyFn = multiply$2({
    calculator: calculator
  });
  return multiplyFn(multiplicand, multiplier);
}

/**
 * Normalize a set of Dinero objects to the highest scale of the set.
 *
 * @param dineroObjects - The Dinero objects to normalize.
 *
 * @returns A new set of Dinero objects.
 */
function normalizeScale() {
  for (var _len = arguments.length, _ref = new Array(_len), _key = 0; _key < _len; _key++) {
    _ref[_key] = arguments[_key];
  }

  var dineroObjects = _ref[0];
  var calculator = dineroObjects[0].calculator;
  var normalizeScaleFn = normalizeScale$1({
    calculator: calculator
  });
  return normalizeScaleFn(dineroObjects);
}

/**
 * Subtract the passed Dinero objects.
 *
 * @param minuend - The Dinero object to subtract from.
 * @param subtrahend - The Dinero object to subtract.
 *
 * @returns A new Dinero object.
 */
function subtract$1() {
  for (var _len = arguments.length, _ref = new Array(_len), _key = 0; _key < _len; _key++) {
    _ref[_key] = arguments[_key];
  }

  var minuend = _ref[0],
      subtrahend = _ref[1];
  var calculator = minuend.calculator;
  var subtractFn = safeSubtract({
    calculator: calculator
  });
  return subtractFn(minuend, subtrahend);
}

/**
 * Format a Dinero object with a custom transformer.
 *
 * @param dineroObject - The Dinero object to format.
 * @param transformer - A transformer function.
 *
 * @returns The object as a formatted string.
 */
function toFormat() {
  for (var _len = arguments.length, _ref = new Array(_len), _key = 0; _key < _len; _key++) {
    _ref[_key] = arguments[_key];
  }

  var dineroObject = _ref[0],
      transformer = _ref[1];
  var calculator = dineroObject.calculator;
  var formatter = toFormat$1({
    calculator: calculator
  });
  return formatter(dineroObject, transformer);
}

/**
 * Get a snapshot of a Dinero object.
 *
 * @param dineroObject - The Dinero object to transform.
 *
 * @returns A snapshot of the object.
 */

var toSnapshot = toSnapshot$1;

/**
 * Get the amount of a Dinero object in units.
 *
 * @param dineroObject - The Dinero object to transform.
 * @param options.digits - The number of fraction digits to round to.
 * @param options.round - The rounding function to use.
 *
 * @returns The amount in units.
 */
function toUnit() {
  for (var _len = arguments.length, _ref = new Array(_len), _key = 0; _key < _len; _key++) {
    _ref[_key] = arguments[_key];
  }

  var dineroObject = _ref[0],
      options = _ref[1];
  var calculator = dineroObject.calculator;
  var toUnitFn = toUnit$1({
    calculator: calculator
  });
  return toUnitFn(dineroObject, options);
}

/**
 * Transform a Dinero object to a new scale.
 *
 * @param dineroObject - The Dinero object to transform.
 * @param newScale - The new scale.
 *
 * @returns A new Dinero object.
 */
function transformScale() {
  for (var _len = arguments.length, _ref = new Array(_len), _key = 0; _key < _len; _key++) {
    _ref[_key] = arguments[_key];
  }

  var dineroObject = _ref[0],
      newScale = _ref[1];
  var calculator = dineroObject.calculator;
  var transformScaleFn = transformScale$1({
    calculator: calculator
  });
  return transformScaleFn(dineroObject, newScale);
}

/**
 * Trim a Dinero object's scale as much as possible, down to the currency exponent.
 *
 * @param dineroObject - The Dinero object which scale to trim.
 *
 * @returns A new Dinero object.
 */
function trimScale() {
  for (var _len = arguments.length, _ref = new Array(_len), _key = 0; _key < _len; _key++) {
    _ref[_key] = arguments[_key];
  }

  var dineroObject = _ref[0];
  var calculator = dineroObject.calculator;
  var trimScaleFn = trimScale$1({
    calculator: calculator
  });
  return trimScaleFn(dineroObject);
}

/**
 * Returns the sum of two numbers.
 *
 * @param augend - The number to add to.
 * @param addend - The number to add.
 *
 * @returns The sum of the two numbers.
 */
var add = function add(augend, addend) {
  return augend + addend;
};

/**
 * Compare two numbers.
 *
 * @param a - The first number to compare.
 * @param b - The second number to compare.
 *
 * @returns Whether the two numbers are equal, or whether the first one is greater or less than the other.
 */

var compare = function compare(a, b) {
  if (a < b) {
    return ComparisonOperator.LT;
  }

  if (a > b) {
    return ComparisonOperator.GT;
  }

  return ComparisonOperator.EQ;
};

/**
 * Returns an decremented number.
 *
 * @param value - The number to decrement.
 *
 * @returns The decremented number.
 */
var decrement = function decrement(value) {
  return value - 1;
};

/**
 * Returns an incremented number.
 *
 * @param value - The number to increment.
 *
 * @returns The incremented number.
 */
var increment = function increment(value) {
  return value + 1;
};

/**
 * Returns the quotient of two numbers with no fractional part.
 *
 * @param dividend - The number to divide.
 * @param divisor - The number to divide with.
 *
 * @returns The quotient of the two numbers.
 */
var integerDivide = function integerDivide(dividend, divisor) {
  return Math.trunc(dividend / divisor);
};

/**
 * Returns the remainder of two numbers.
 *
 * @param dividend - The number to divide.
 * @param divisor - The number to divide with.
 *
 * @returns The remainder of the two numbers.
 */
var modulo = function modulo(dividend, divisor) {
  return dividend % divisor;
};

/**
 * Returns the product of two numbers.
 *
 * @param multiplicand - The number to multiply.
 * @param multiplier - The number to multiply with.
 *
 * @returns The product of the two numbers.
 */
var multiply = function multiply(multiplicand, multiplier) {
  return multiplicand * multiplier;
};

/**
 * Returns an number to the power of an exponent.
 *
 * @param base - The base number.
 * @param exponent - The exponent to raise the base to.
 *
 * @returns The base to the power of the exponent.
 */
var power = function power(base, exponent) {
  return Math.pow(base, exponent);
};

/**
 * Returns the difference between two numbers.
 *
 * @param minuend - The number to subtract from.
 * @param subtrahend - The number to subtract.
 *
 * @returns The difference of the two numbers.
 */
var subtract = function subtract(minuend, subtrahend) {
  return minuend - subtrahend;
};

/**
 * Transforms an value to a number.
 *
 * @param input - The value to transform.
 *
 * @returns The transformed value.
 */
var toNumber = function toNumber(input) {
  return input;
};

/**
 * Return zero as a number.
 *
 * @returns Zero as a number.
 */
function zero() {
  return 0;
}

var calculator = {
  add: add,
  compare: compare,
  decrement: decrement,
  increment: increment,
  integerDivide: integerDivide,
  modulo: modulo,
  multiply: multiply,
  power: power,
  subtract: subtract,
  toNumber: toNumber,
  zero: zero
};

/* eslint-disable functional/no-expression-statement */
/**
 * Create a Dinero object.
 *
 * @param options.amount - The amount in minor currency units.
 * @param options.currency - The currency.
 * @param options.scale - The number of decimal places to represent.
 *
 * @returns The created Dinero object.
 */

var dinero = createDinero({
  calculator: calculator,
  onCreate: function onCreate(_ref) {
    var amount = _ref.amount,
        scale = _ref.scale;
    assert(Number.isInteger(amount), INVALID_AMOUNT_MESSAGE);
    assert(Number.isInteger(scale), INVALID_SCALE_MESSAGE);
  }
});

exports.add = add$1;
exports.allocate = allocate;
exports.convert = convert;
exports.dinero = dinero;
exports.equal = equal;
exports.greaterThan = greaterThan;
exports.greaterThanOrEqual = greaterThanOrEqual;
exports.hasSubUnits = hasSubUnits;
exports.haveSameAmount = haveSameAmount;
exports.haveSameCurrency = haveSameCurrency;
exports.isNegative = isNegative;
exports.isPositive = isPositive;
exports.isZero = isZero;
exports.lessThan = lessThan;
exports.lessThanOrEqual = lessThanOrEqual;
exports.maximum = maximum;
exports.minimum = minimum;
exports.multiply = multiply$1;
exports.normalizeScale = normalizeScale;
exports.subtract = subtract$1;
exports.toFormat = toFormat;
exports.toSnapshot = toSnapshot;
exports.toUnit = toUnit;
exports.transformScale = transformScale;
exports.trimScale = trimScale;
//# sourceMappingURL=index.development.js.map
